-- Utils

local Odyssey = Apollo.GetAddon("Odyssey")

local tinsert = table.insert
local pairs = pairs
local floor = math.floor
local round = function(val) return math.floor(val + .5) end
local type = type

function Odyssey.formatSeconds(seconds)
	local minutes = floor(seconds / 60)
	seconds = floor(seconds % 60)
	return ("%02d:%02d"):format(minutes, seconds)
end

function Odyssey.numberShortFormat(num)
  if Odyssey.settings.showShortNumber then
    if round(num) > 1000000 then
      local format = "%.1fm"
      return format:format(tostring(round(num) / 1000000))
    elseif round(num) > 1000 then
      local format = "%.1fk"
      return format:format(tostring(round(num) / 1000))
    else
      return tostring(round(num))
    end
  else
    return tostring(round(num))
  end
end

function Odyssey.numberFormat(num)
	local str = tostring(round(num))
	if Odyssey.settings.showShortNumber then
	  return Odyssey.numberShortFormat(num)
	else
	 local formatted = str:reverse():gsub("(%d%d%d)","%1,"):reverse()
	 return str:len() % 3 == 0 and formatted:sub(2) or formatted
	end
end

function Odyssey.colorize(text, fromHex, toHex)
	local colored = {}
	local len = text:len() - 1


	local from = {
		r = bit.rshift(fromHex, 16),
		g = bit.band(bit.rshift(fromHex, 8), 0xff),
		b = bit.band(fromHex, 0xff)
	}

	local to = {
		r = bit.rshift(toHex, 16),
		g = bit.band(bit.rshift(toHex, 8), 0xff),
		b = bit.band(toHex, 0xff)
	}
	
	local step = {
		r = (to.r - from.r) / len,
		g = (to.g - from.g) / len,
		b = (to.b - from.b) / len
	}

	for char in text:gmatch(".") do
		tinsert(colored, ("<P TextColor=\"ff%02x%02x%02x\">%s</P>"):format(from.r, from.g, from.b, char))
		from.r = from.r + step.r
		from.g = from.g + step.g
		from.b = from.b + step.b
	end

	return table.concat(colored)
end

function Odyssey.deepcopy(orig)
    local orig_type = type(orig)
    local copy
    if orig_type == 'table' then
        copy = {}
        for orig_key, orig_value in next, orig, nil do
            copy[Odyssey.deepcopy(orig_key)] = Odyssey.deepcopy(orig_value)
        end
        setmetatable(copy, Odyssey.deepcopy(getmetatable(orig)))
    else -- number, string, boolean, etc
        copy = orig
    end
    return copy
end